/*
 * mm_cp.c - Copy a file using mmap'd memory.
 *
 * Author: Original author unknown (modified by Randy L. Pearson)
 * Date:   unknown
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <stdio.h>

#define FILE_MODE (S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH)

void err_quit( const char * msg )
{
	fprintf( stderr, msg );
	exit(1);
}

void err_sys( const char * msg )
{
	perror( msg );
	exit(1);
}

int main (int argc, char *argv[])
{
	int fdin, fdout;
	char *src, *dst;
	struct stat statbuf;

	if (argc != 3)
		err_quit ("usage: mm_cp <fromfile> <tofile>");

	/* open the input file */
	if ((fdin = open (argv[1], O_RDONLY)) < 0)
		err_sys(argv[1]);

	/* open/create the output file */
	if ((fdout = open (argv[2], O_RDWR | O_CREAT | O_TRUNC, FILE_MODE)) < 0)
		err_sys(argv[2]);

	/* find size of input file */
	if (fstat (fdin,&statbuf) < 0)
		err_sys("could not read input file status");

	/* go to the location corresponding to the last byte */
	if (lseek (fdout, statbuf.st_size - 1, SEEK_SET) == -1)
		err_sys("could not seek on output file");
 
	/* write a dummy byte at the last location */
	if (write (fdout, "", 1) != 1)
		err_sys("could not write to output file");

	/* mmap the input file */
	if ((src = mmap (0, statbuf.st_size, PROT_READ, MAP_SHARED, fdin, 0)) == (caddr_t) -1)
		err_sys("mmap for input file failed");

	/* mmap the output file */
	if ((dst = mmap (0, statbuf.st_size, PROT_READ | PROT_WRITE, MAP_SHARED, fdout, 0)) == (caddr_t) -1)
		err_sys("mmap for output file failed");

	/* this copies the input file to the output file */
 	memcpy (dst, src, statbuf.st_size);

	/* Exiting should automatically close the files and unmap the memory */
	return 0;
}


