/*
 * options.c -- Demonstrates using the getopt() function.
 *
 * Author: Randy L. Pearson
 * Date:   22 Jun 2004
 */
 
#include <unistd.h>
#include <string.h>
#include <stdio.h>

int main( int argc, char * argv[] )
{
	int option, i;
	
	/* Show arguments before processing */
	printf( "Before processing command line - \n" );
	printf( "Number of arguments: %d\n", argc );
	for( i = 0; i < argc; i++ )
		printf( "Argument[%d]: %s\n", i, argv[i] );
	
	/* Process options */
	printf( "\nProcessing command line...\n\n" );
	while( (option = getopt(argc, argv, "hlvi:o::")) != -1 )
	{
		switch( option )
		{
		case 'h':
		case 'l':
		case 'v':
			printf( "Found option: %c\n", option );
			break;
		case 'i':
			printf( "-i argument: %s\n", optarg );
			break;
		case 'o':
			if( optarg != 0 )
				printf( "-o optional argument: %s\n", optarg );
			else
				printf( "-o option (no argument)\n" );
			break;
		case '?':
			printf( "Unrecognized option or missing argument detected.\n" );
			break;
		case ':':
			printf( "Option is missing a required argument.\n" );
			break;
		default:
			printf( "getopt() returned a %c\n", option );
		}
	}
	
	/* Show argument after processing. */
	printf( "\nAfter processing command line - \n" );
	printf( "Number of arguments: %d\n", argc );
	for( i = 0; i < argc; i++ )
		printf( "Argument[%d]: %s\n", i, argv[i] );
		
	/* Show the values where optind points */
	printf( "\nVariable optind: %d\n", optind );
	for( i = optind; i < argc; i++ )
		printf( "Option[i]: %s\n", argv[i] );
		
	return 0;
}
