/*
 * showerr.c -- Simple program to display error values/strings.
 *
 * Author: Randy L. Pearson
 * Date:   22 Jun 2004
 */
 
#include <string.h>
#include <stdio.h>
#include <errno.h>

/* Not all system define this code in <errno.h> */
#ifndef ENOTCON
#define ENOTCON 107
#endif

struct ErrorCode
{
	int	value;
	const char *name;
};

struct ErrorCode codes[] =
{
	{ E2BIG,			"E2BIG" },
	{ EACCES,			"EACCES" },
	{ EADDRINUSE,		"EADDRINUSE" },
	{ EADDRNOTAVAIL,	"EADDRNOTAVAIL" },
	{ EAFNOSUPPORT,		"EAFNOSUPPORT" },
	{ EAGAIN,			"EAGAIN" },
	{ EALREADY,			"EALREADY" },
	{ EBADF,			"EBADF" },
	{ EBADMSG,			"EBADMSG" },
	{ EBUSY,			"EBUSY" },
	{ ECANCELED,		"ECANCELED" },
	{ ECHILD,			"ECHILD" },
	{ ECONNABORTED,		"ECONNABORTED" },
	{ ECONNREFUSED,		"ECONNREFUSED" },
	{ ECONNRESET,		"ECONNRESET" },
	{ EDEADLK,			"EDEADLK" },
	{ EDESTADDRREQ,		"EDESTADDRREQ" },
	{ EDOM,				"EDOM" },
	{ EDQUOT,			"EDQUOT" },
	{ EEXIST, 			"EEXIST" },
	{ EFAULT,			"EFAULT" },
	{ EFBIG,			"EFBIG" },
	{ EHOSTUNREACH,		"EHOSTUNREACH" },
	{ EIDRM,			"EIDRM" },
	{ EILSEQ,			"EILSEQ" },
	{ EINPROGRESS,		"EINPROGRESS" },
	{ EINTR,			"EINTR" },
	{ EINVAL,			"EINVAL" },
	{ EIO,				"EIO" },
	{ EISCONN,			"EISCONN" },
	{ EISDIR,			"EISDIR" },
	{ ELOOP,			"ELOOP" },
	{ EMFILE,			"EMFILE" },
	{ EMLINK,			"EMLINK" },
	{ EMSGSIZE,			"EMSGSIZE" },
	{ EMULTIHOP,		"EMULTIHOP" },
	{ ENAMETOOLONG,		"ENAMETOOLONG" },
	{ ENETDOWN,			"ENETDOWN" },
	{ ENETRESET,		"ENETRESET" },
	{ ENETUNREACH,		"ENETUNREACH" },
	{ ENFILE,			"ENFILE" },
	{ ENOBUFS,			"ENOBUFS" },
	{ ENODATA,			"ENODATA" },
	{ ENODEV,			"ENODEV" },
	{ ENOENT,			"ENOENT" },
	{ ENOEXEC,			"ENOEXEC" },
	{ ENOLCK,			"ENOLCK" },
	{ ENOLINK,			"ENOLINK" },
	{ ENOMEM,			"ENOMEM" },
	{ ENOMSG,			"ENOMSG" },
	{ ENOPROTOOPT,		"ENOPROTOOPT" },
	{ ENOSPC,			"ENOSPC" },
	{ ENOSR,			"ENOSR" },
	{ ENOSTR,			"ENOSTR" },
	{ ENOSYS,			"ENOSYS" },
	{ ENOTCON,			"ENOTCON" },
	{ ENOTDIR,			"ENOTDIR" },
	{ ENOTEMPTY,		"ENOTEMPTY" },
	{ ENOTSOCK,			"ENOTSOCK" },
	{ ENOTSUP,			"ENOTSUP" },
	{ ENOTTY,			"ENOTTY" },
	{ ENXIO,			"ENXIO" },
	{ EOPNOTSUPP,		"EOPNOTSUPP" },
	{ EOVERFLOW,		"EOVERFLOW" },
	{ EPERM,			"EPERM" },
	{ EPIPE,			"EPIPE" },
	{ EPROTO,			"EPROTO" },
	{ EPROTONOSUPPORT,	"EPROTONOSUPPORT" },
	{ EPROTOTYPE,		"EPROTOTYPE" },
	{ ERANGE,			"ERANGE" },
	{ EROFS,			"EROFS" },
	{ ESPIPE,			"ESPIPE" },
	{ ESRCH,			"ESRCH" },
	{ ESTALE,			"ESTALE" },
	{ ETIME,			"ETIME" },
	{ ETIMEDOUT,		"ETIMEDOUT" },
	{ ETXTBSY,			"ETXTBSY" },
	{ EWOULDBLOCK,		"EWOULDBLOCK" },
	{ EXDEV,			"EXDEV" },
};

int main( int argc, char *argv[] )
{
	int numCodes = sizeof(codes)/sizeof(codes[0]);
	int longest = 0;
	int largest = 0;
	int i, j;
	
	/* Loop and find longest error code string for later usage */
	for( i = 0; i < numCodes; ++i )
	{
		int len = strlen(codes[i].name);
		longest = (len > longest ? len : longest);
		largest = (codes[i].value > largest ? codes[i].value : largest);
	}
		
	/* Loop and show error code (value): message */
	for( i = 0; i <= largest; i++ )
	{
		for( j = 0; j < numCodes; j++ )
			if( i == codes[j].value )
				break;
				
		const char * errstr = strerror( i );
		char message[255];
		
		if( j < numCodes )
			sprintf( message, "%s (%d) ...", codes[j].name, codes[j].value );
		else
			sprintf( message, "??? (%d) ...", i );
			
		while( strlen(message) < longest+10 )
			strcat( message, "." );
		printf( "%s %s\n", message, errstr );
	}
	
	return 0;
}
