/******
 *
 * process.c - This file defines most of the functions that
 *             process the input data and generate the results.
 *
 * Author: Randy L. Pearson
 *
 * Revision History -
 *
 * 10 Jun 2004 - Created it.
 *
 ******/
 
#include <stdio.h>
#include <string.h>
#include "smtp-blocker.h"

#define MAXLEN	255

#define SPF_REJECT_START	"Local SPF Rejects:"

/**
 * lookup: performs a WHOIS lookup for the ip address
 *         and sends network information to output.
 */
 
static void lookup( char *email, char *ip, char *host )
{
	char command[MAXLEN];
	FILE *pCmd;
	
	strcpy( command, "whois " );
	strcat( command, ip );
	
	if( verbose )
		fprintf( stderr, "Executing: %s\n", command );
	
	if( (pCmd = popen( command, "r" )) == 0 )
	{
		perror( command );
		return;
	}
	
	while( !feof(pCmd) )
	{
		fgets( command, MAXLEN, pCmd );
		fprintf( dest, command );
	}
	
	pclose(pCmd);
}

/**
 * analyze: Attempts to divide the input string into
 *          e-mail address, ip address and hostname.
 *          If successful, calls lookup(), otherwise
 *          just returns (ignores line).
 */
static void analyze( char *data )
{
	char *email = data, *ip, *hostname, *split;
	
	if( (split = strstr( data, "-->" )) == 0 )
		return;
	
	/*
	 * Make addr point to the start of the e-mail address
	 * and then terminate the address string
	 */
	while( *email == ' ' || *email == '\t' )
		email++;
	*(split-1) = 0;
	
	/*
	 * Find and terminate the IP address.
	 */
	ip = split+4;
	if( (split = strchr( ip, ' ' )) == 0 )
		return;
		
	*split = 0;
	
	/*
	 * And repeat for the hostname part
	 */
	hostname = split+2;
	if( (split = strchr( hostname, ']' )) == 0 )
		return;
	*split = 0;
	
	lookup( email, ip, hostname );
}

/**
 * process_data: Reads from the input until end of file or
 *               we find the end of the SPF section.
 **/
void process_data()
{
	int processing = 0;
	char line[MAXLEN] = {0};
	
	while( !feof(src) )
	{
		fgets( line, MAXLEN, src );
		
		/* Remove newline character from line */
		if( strchr( line, '\n' ) )
			*strchr( line, '\n') = 0;
			
		/* Ignore blank lines */
		if( strlen(line) == 0 )
			continue;
			
		if( !processing )
		{
			/* Not processing yet, so look for start of the SPF section */
			if( strncmp( line, SPF_REJECT_START, strlen(SPF_REJECT_START) ) == 0 )
			{
				processing = 1;
				if( verbose )
					fprintf( stderr, "Found tag \"%s\".  Beginning analysis phase.\n",
										SPF_REJECT_START );
			}
			continue;
		}

		/*
		 * Stop processing at first line that does not begin
		 * with a space or tab character.
		 */
		if( line[0] != ' ' && line[0] != '\t' )
		{
			if( verbose )
				fprintf( stderr, "End of SPF section detected.  Exiting...\n" );
			break;
		}
			
		analyze( line );
	}
}
