/******
 *
 * process.c - This file defines most of the functions that
 *             process the input data and generate the results.
 *
 * Author: Randoy L. Pearson
 *
 * Revision History -
 *
 * 10 Jun 2004 - Created it.
 *
 ******/
 
#include <stdio.h>
#include <string.h>
#include <pcre.h>
#include "smtp-blocker.h"

#define MAXLEN	255

#define SPF_REJECT_START	"Local SPF Rejects:"

/**
 * lookup: performs a WHOIS lookup for the ip address
 *         and sends network information to output.
 */
 
static void lookup( char *email, char *ip, char *host )
{
	char command[MAXLEN];
	FILE *pCmd;
	
	strcpy( command, "whois " );
	strcat( command, ip );
	
	if( verbose )
		fprintf( stderr, "Executing: %s\n", command );
	
	if( (pCmd = popen( command, "r" )) == 0 )
	{
		perror( command );
		return;
	}
	
	while( !feof(pCmd) )
	{
		fgets( command, MAXLEN, pCmd );
		fprintf( dest, command );
	}
	
	pclose(pCmd);
}

static void update_db( const char *user, const char *domain, const char *ip, const char *host )
{
	fprintf( stderr, "TODO: Add record to database -- %s/%s/%s/%s\n", user, domain, ip, host );
}

/**
 * analyze: Attempts to divide the input string into
 *          e-mail address, ip address and hostname.
 *          If successful, calls lookup(), otherwise
 *          just returns (ignores line).
 */
static void analyze( char *data )
{
	static const char *pattern = "\\t+(.*)@(.*) --> (\\d+\\.\\d+\\.\\d+\\.\\d+) \\[(.*)\\]";
	pcre *re;
	const char *errmsg;
	int erroffset;
	int result;
	int ovector[30];
	
	re = pcre_compile( pattern, 0, &errmsg, &erroffset, NULL);
	if( re == NULL )
	{
		fprintf( stderr, "RegEx error: %s at %s\n", errmsg, &pattern[erroffset] );
		return;
	}
	
	result = pcre_exec( re, NULL, data, strlen(data), 0, 0, ovector, 30 );
	if( result >= 5 )
	{
		char user[255] = {0};
		char domain[255] = {0};
		char ip[255] = {0};
		char host[255] = {0};
		
		strncpy( user, data + ovector[2], ovector[3]-ovector[2] );
		strncpy( domain, data + ovector[4], ovector[5]-ovector[4] );
		strncpy( ip, data + ovector[6], ovector[7]-ovector[6] );
		strncpy( host, data + ovector[8], ovector[9]-ovector[8] );
		
		update_db( user, domain, ip, host );
	}
	else
	{
		if( verbose )
			fprintf( stderr, "Skipping---%s\n", data );
	}
}

/**
 * process_data: Reads from the input until end of file or
 *               we find the end of the SPF section.
 **/
void process_data()
{
	int processing = 0;
	char line[MAXLEN] = {0};
	
	while( !feof(src) )
	{
		fgets( line, MAXLEN, src );
		
		/* Remove newline character from line */
		if( strchr( line, '\n' ) )
			*strchr( line, '\n') = 0;
			
		/* Ignore blank lines */
		if( strlen(line) == 0 )
			continue;
			
		if( !processing )
		{
			/* Not processing yet, so look for start of the SPF section */
			if( strncmp( line, SPF_REJECT_START, strlen(SPF_REJECT_START) ) == 0 )
			{
				processing = 1;
				if( verbose )
					fprintf( stderr, "Found tag \"%s\".  Beginning analysis phase.\n",
										SPF_REJECT_START );
			}
			continue;
		}

		/*
		 * Stop processing at first line that does not begin
		 * with a space or tab character.
		 */
		if( line[0] != ' ' && line[0] != '\t' )
		{
			if( verbose )
				fprintf( stderr, "End of SPF section detected.  Exiting...\n" );
			break;
		}
			
		analyze( line );
	}
}
