/*
 * timestuff.c -- Demonstrate playing with time functions.
 *
 * Author: Randy L. Pearson
 * Date:   10 Feb 2003
 */

#include <stdio.h>
#include <time.h>


time_t		timeStart, timeNow;
struct tm	tmStart, tmNow;

void setup()
{
	time(&timeStart);
	tmStart = *localtime(&timeStart);
}

void update()
{
	time(&timeNow);
	tmNow = *localtime(&timeNow);
}

void show()
{
	char buffer[80];

	strftime( buffer, sizeof buffer, "%c", &tmStart );
	printf( "Start time: %s\n", buffer );
	strftime( buffer, sizeof buffer, "%c", &tmNow );
	printf( "Current time: %s\n", buffer );
}

void wastetime()
{
	printf( " \b" );
	fflush( stdout );
}

void donothing()
{
	// Doing nothing here
}

void play()
{
	int i, j, k;
	time_t theTime, theTime2;
	double loops;

	puts("Playing with simple output...");

	for( i = 0; i < 10; i++ )
	{
		printf( "Hello there." );
		fflush( stdout );
		for( j = 0; j < 12; j++ )
		{
			printf( "\b \b" );
			fflush( stdout );
			for( k = 0; k < 1000000; k++ )
				donothing();
		}
	}
	printf( "\n" );
	
	puts( "Playing with the time() function..." );

	for( i = 0; i < 10; i++ )
	{
		time( &theTime );
		printf( "Seconds since Epoch: %ld\n", theTime );
		sleep(1);
	}

	puts( "Timing a do-nothing loop...please wait" );
	time( &theTime );

	loops = 0;
	do {
		donothing();
		loops += 1;
		time( &theTime2 );
	} while( theTime2 - theTime < 10 );

	printf( "\nPerformed %2.0f do-nothings in %d seconds\n",
		loops, theTime2 - theTime );
	printf( "Loops per second: %2.1f\n",
		loops / (double) (theTime2 - theTime) );

	puts( "Timing the waste-time function...please wait" );
	time( &theTime );

	loops = 0;
	do {
		wastetime();
		loops += 1;
		time( &theTime2 );
	} while( theTime2 - theTime < 10 );

	
	printf( "\nPerformed %2.0f waste-time functions in %d seconds\n",
		loops, theTime2 - theTime );
	printf( "Loops per second: %2.1f\n",
		loops / (double) (theTime2 - theTime) );
}

int main()
{
	setup();
	puts("Wasting time by playing with time...:-)");
	update();
	show();
	play();
	update();
	show();
	exit(0);
}
