/*
 * userinfo.c -- Demonstrate various UID/GID functions.
 *
 * Author: Randy L. Pearson
 * Date:   22 Jun 2004
 */
 
#include <unistd.h>
#include <stdio.h>
#include <pwd.h>
#include <sys/types.h>

void show_passwd( const struct passwd * info );
void show_all_passwds();

int main()
{
	uid_t uid = getuid();
	uid_t euid = geteuid();
	gid_t gid = getgid();
	gid_t egid = getegid();
	char *login = getlogin();
	char login_name[50];
	char cuser_name[L_cuserid];
	
	printf( "You are logged in as: %s\n", login );
	printf( "Your UID is: %d\n", uid );
	printf( "Your GID is: %d\n", gid );
	printf( "Your effective UID is: %d\n", euid );
	printf( "Your effective GID is: %d\n", egid );
	
	if( getlogin_r( login_name, sizeof(login_name) ) )
		perror( "getlogin_r failed" );
	else
		printf( "getlogin_r returned: %s\n", login_name );
		
	cuserid( cuser_name );
	printf( "cuserid returned: %s\n", cuser_name );
	
	printf( "Password info for UID %d:\n", uid );
	show_passwd( getpwuid( uid ) );
	
	printf( "Password info for EUID: %d\n", euid );
	show_passwd( getpwuid( euid ) );
	
	printf( "\nAll passwd entries:\n\n" );
	show_all_passwds();
	
	return 0;
}

void show_passwd( const struct passwd *info )
{
	if( info )
	{
		printf( "Name: %s", info->pw_name );
		printf( "\tPassword: %s", info->pw_passwd );
		printf( "\tUID: %d", info->pw_uid );
		printf( "\tGID: %d", info->pw_gid );
		printf( "\tReal Name: %s", info->pw_gecos );
		printf( "\tHome: %s", info->pw_dir );
		printf( "\tShell: %s\n", info->pw_shell );
	}
}

void show_all_passwds()
{
	const struct passwd *pwd;
	
	while( (pwd = getpwent()) )
		show_passwd( pwd );
}
